function h= ir_room( x_spker, x_sensor, room_dim, beta, N, dim_of_images, lp_parameter )
%
% June 2004
%
% simulation of acoustic room impulse response 
% (using the image method modified by Patrick Peterson)
%
% h= ir_room( x_spker, x_sensor, room_dim, beta, N, dim_of_images, lp_parameter )
%
% x_spkr= [x,y,z]' is the speaker location
% x_sensor= [x,y,z]' is the sensor location
% room_dim= [Lx,Ly,Lz] is the room size.
% beta= [ beta_x1 beta_y1 beta_z1; beta_x2 beta_y2 beta_z2 ]
%   contains the reflection coefficients of the walls.
% N is the length of the room impulse response
% dim_of_images is the no. of images used
%
% lp_parameter= [ Mw Wc ]; where
%    Mw is the ideal lowpass filter length (80 by default)
%    Wc is the cut-off frequency (0.9 by default)
%
% Comment: the previous function 'ir_room' appears to be not very accurate
%       when used to simulate TDOA problems.

if nargin <=6,
    Mw= 80; %filter width
    Wc= 0.9;    %cut-off frequency
else
    Mw= lp_parameter(1); Wc= lp_parameter(2);
end;

%--------
Rp= zeros(3,8);
for p=1:8
   str= dec2bin(p-1,3);
   v= [ str2num(str(1))*2-1 str2num(str(2))*2-1 str2num(str(3))*2-1 ]';
   Rp(:,p)= x_spker+ v.*x_sensor;
end;

%--------
h= zeros(N,1);
for p=1:8
   str= dec2bin(p-1,3); v= [ str2num(str(1)); str2num(str(2)); str2num(str(3)) ];
   for n=-dim_of_images:1:dim_of_images
      for l= -dim_of_images:1:dim_of_images
         for m= -dim_of_images:dim_of_images
            Rr= 2*[ n; l; m].*room_dim;
            D= norm(Rp(:,p)+Rr);
            if D < N,
                k= max(0,ceil(-Mw/2+D)):1:min(N-1,floor(Mw/2+D)); k= k';
                h(k+1)= h(k+1)+ ideal_lp(Wc,Mw,k-D)*(beta(1,1)^abs(n-v(1)))*(beta(2,1)^abs(n))* ...
               	(beta(1,2)^abs(l-v(2)))*(beta(2,2)^abs(l))*(beta(1,3)^abs(m-v(3)))*...
                (beta(2,3)^abs(m))/(4*pi*D);
            end;
         end;
      end;
   end;
end;


%----
function f= ideal_lp(Wc,Mw,n)

f= .5*(1+cos(2*pi*n/Mw)).*sinc(Wc*n);
