function d_wass= wasserstein_dist(X,Y)
% compute the Wasserstein distances of two finite sets
% function d_wass= wasserstein_dist(X,Y)
% where X= [ x_1, ..., x_n ] and Y= [ y_1, ..., y_m ]

q= size(X,1);
n= size(X,2); m= size(Y,2);
D= zeros(n,m);
if q> 1,
    for j=1:m
        D(:,j)= sum( ( repmat(Y(:,j),[1 n])- X ).^2 )';
    end;
else
    for j=1:m
        D(:,j)= (( repmat(Y(:,j),[1 n])- X ).^2)';
    end;
end;
if n==1 | m==1, % for n=1 or m=1 the computation of Wasserstein distance is trivial
    d_wass= sqrt(sum(D)/max([n,m]));
else
    %--- run linear programming for computing the Wasserstein distance
    ct= D(:); At= [ kron(ones(1,m),eye(n)); kron(eye(m),ones(1,n)) ];
    bt= [ ones(n,1)/n; ones(m,1)/m ]; Kt.l= [n*m]; pars.fid= 0;
    if (n+m) < n*m %is no. of constraints less than problem dimension
        [xt,yt]= sedumi(At,bt,ct,Kt,pars);
    else % we need to do a bit reformulation to make no. of constraints < problem dim.
        r= rank(At);
        [Q,R]= qr(At); Q1= Q(:,1:r); R1= R(1:r,:);
        [xt,yt]= sedumi(R1,Q1'*bt,ct,Kt,pars);
    end;
    d_wass= sqrt(xt'*ct);   
end;