%To start with, let's make it simple: 1 speaker; 2 microphone pairs

fcode='test2';

fs= 8000;     %sampling rate
K_spkr= 2;           %no. of speakers
Nframe= 512*8;   %FFT frame length
block_overlap= .5;   %block overlap factor for computing TDOAs
P= 4;           %no. of TDOA microphone pairs

snr_db= 20;
snr= 10^(snr_db/10);
Ts= 1/fs;


%--- parameters associated with room impulse responses
c= 344;			%speed of sound
rlim= 5;			%represent (but not exactly) the no. of multipaths
room_dim= [ 3; 3; 2.5 ];    %room size
beta= [ 0.85 0.85 0.75; 0.85 0.85 0.5 ];	%reflection coefficients of walls
%beta= 1e-6*ones(2,3);

%x_s= [ 1; 2; 1.5 ];  %speaker coordinate
x_s= zeros(3,K_spkr);
x_s(:,1)= [ 1.2; 1.8; 1.5 ];
x_s(:,2)= [ 2; 2.5; 1.8 ];
x_sensor= zeros(3,2,P);     %microphone pair coordinates
x_sensor(:,1,1)= [ 1.25; 0.01; 2.4 ]; x_sensor(:,2,1)= [ 1.75; 0.01; 2.4 ];
x_sensor(:,1,2)= [ 0.01; 1.25; 2.4 ]; x_sensor(:,2,2)= [ 0.01; 1.75; 2.4 ];
x_sensor(:,1,3)= [ 1.25; 2.99; 2.4 ]; x_sensor(:,2,3)= [ 1.75; 2.99; 2.4 ];
x_sensor(:,1,4)= [ 2.99; 1.25; 2.4 ]; x_sensor(:,2,4)= [ 2.99; 1.75; 2.4 ];


dR= c*Ts;
max_d= ceil(.5/dR);       %max. no. of TDOA samples (.5 is the microphone pair spacing)


%---
h_len= 8192;      %length of room impulses responses (interpolated)

if exist(['roomir_',fcode,'.mat'],'file'), gen_ir= 'n'; 
else gen_ir= 'y'; end;

if gen_ir=='y',
    disp(['Computing the room impulse responses...']);
    h= zeros(h_len,2,P,K_spkr);
    for k=1:K_spkr
        for p=1:P
            h(:,1,p,k)= ir_room2(x_s(:,k)/dR,x_sensor(:,1,p)/dR,room_dim/dR,beta,h_len,rlim);
            h(:,2,p,k)= ir_room2(x_s(:,k)/dR,x_sensor(:,2,p)/dR,room_dim/dR,beta,h_len,rlim);
        end;
    end;
    save( ['roomir_',fcode], 'h' );
else
    load(['roomir_',fcode]);
end;

%--- load speech signals
%st= wavread('SA1');	st= decimate(st,2); % first speaker speech
st= randn(20*Nframe,1);

N= length(st);
N= ceil(N/Nframe)*Nframe; 	%modify data length

s= zeros(N,K_spkr);

s(1:length(st),1)= st/(norm(st)/sqrt(N));

%st2= wavread('SA2'); st2= decimate(st2,2); %second speaker speech
st2= randn(N,1);

if length(st2)> N,
    s(:,2)= st2(1:N);
else
    s(1:length(st2),2)= st2/(norm(st2)/sqrt(N));
end;

