%==== Multi-sensor 1 target Bayesian filtering:
% run this routine after you call 'declare_problem_single' and signal generation


%---
hat_N= zeros(K,1);      %(soft) estimate of the number of targets
hat_x= zeros(x_dim,K);
particle_X= cell(K,1);  %cell for storing all particles
particle_W= cell(K,1);  %cell for storing the update weights
Xk= [];
Wk= ones(Ns,1)/Ns;

for k=1:K
    time_start= cputime;
    %--- particle generation & multisensor weight update
    Xk_old= Xk;
    no_notempty_Xk= size(Xk_old,2);
    Wk_old_notempty= Wk(1:no_notempty_Xk);
    Wk_old_empty= Wk((no_notempty_Xk+1):Ns);
    Xk= []; Wk= [];
    %death process
    rv= rand(no_notempty_Xk,1);
    idx= find(rv <= P_death); idx_c= find(rv > P_death);
    Xk= gen_newstate(model,Xk_old(:,idx_c)); Wk= Wk_old_notempty(idx_c);
    %birth
    rv= rand((Ns-no_notempty_Xk),1);
    idx2= find( rv<= P_birth ); idx2_c= find( rv > P_birth );
    Xk= [ Xk gen_birthstate(model,length(idx2)) ];
    Wk= [ Wk; Wk_old_empty(idx2) ];
    %arrange weights of empty X to the end
    Wk= [ Wk; Wk_old_notempty(idx); Wk_old_empty(idx2_c) ];
    %--- weight update
    no_notempty_Xk= size(Xk,2);
    for q=1:Q
        Zk_len= size(Z{q,k},2);
        f_zx= zeros(1,no_notempty_Xk);
        for i=1:Zk_len
            f_zx= f_zx+ compute_likelihood(model,Z{q,k}(:,i),Xk,q);
        end;
        Wk(1:no_notempty_Xk)= Wk(1:no_notempty_Xk).* ...
            ( (1-P_D) + P_D*f_zx'/(lambda_c/prod(range_c(:,2)-range_c(:,1))) );
    end;
    %--- weight normalization and resampling
    Wk= Wk/sum(Wk);
    idx= resample(Wk,Ns);
    i1= find(idx<= no_notempty_Xk); i2= find(idx> no_notempty_Xk);
    idx_notempty= idx(i1); %idx_empty= idx(i2);
    Xk= Xk(:,idx_notempty);
    %Wk= [ Wk(idx_notempty); Wk(idx_empty) ];
    Wk= ones(Ns,1)/Ns;
    %--- estimate from particles
    hat_N(k)= no_notempty_Xk/Ns;
    hat_x(:,k)= sum(Xk')'/size(Xk,2);
    particle_X{k}= Xk; particle_W{k}= Wk;
end;

return
    


Wk_resample= 0;
for k=1:K
    time_start= cputime;
    %---particle generation
    tilde_Xk= [ gen_newstate(model,tilde_Xk) gen_birthstate(model,Jk) ];
    Wk_predict= [ Wk_resample*(1-P_death); ones(Jk,1)*lambda_b/Jk ];
    %---mulitsensor weight update
    for q=1:Q
        Zk_len= size(Z{q,k},2);
        Gk= zeros(Zk_len,Lk+Jk);  %likelihood function matrix
        Ck_z= zeros(Zk_len,1);
        Uk_z= zeros(Zk_len,1);  %Uk_Z is denoted to be the denominator of the update eqn.
        for i=1:Zk_len
            Gk(i,:)= compute_likelihood(model,Z{q,k}(:,i),tilde_Xk,q);
            Ck_z(i)= P_D*(Gk(i,:)*Wk_predict);
            Uk_z(i)= lambda_c/prod(range_c(:,2)-range_c(:,1))+Ck_z(i);
            %note that we assume z always lies in the clutter region
        end;
        Wk_update= (P_D*(Gk'*(1./Uk_z))+ (1-P_D)).*Wk_predict;
        Wk_predict= Wk_update; %w_update for current sensor is w_predict for next sensor
    end;
    particle_W{k}= Wk_update;
    %---resampling
    hat_N(k)= sum(Wk_update);
    if hat_N(k)> 1e-12, %resampling
        Lk= min(ceil(hat_N(k))*rho,Lmax);
        idx= resample(Wk_update/hat_N(k),Lk);
        tilde_Xk_resample= tilde_Xk(:,idx);
        tilde_Xk= tilde_Xk_resample; Wk_resample= hat_N(k)*ones(Lk,1)/Lk;
    else %no resampling; everything reinitialize
        Lk= 1; tilde_Xk= zeros(x_dim,1); Wk_resample= 0;
    end;
    particle_X{k}= tilde_Xk;
    %--- display
    disp(['time= ',num2str(k),'  Estimated N(k)= ',num2str(hat_N(k)),...
            ' True N(k)= ',num2str(N_true(k)),'  Lk= ',num2str(Lk), ...
            '  CPU time= ',num2str(cputime-time_start)]);
end;
        