%To start with, let's make it simple: 1 speaker; 2 microphone pairs

fcode='test1';

fs= 8000;     %sampling rate
K= 1;           %no. of speakers
Nframe= 2048/2;   %FFT frame length
block_overlap= 1;   %block overlap factor for computing TDOAs
                    %always 1 for now
L= 25;              %no. of frames
N= L*Nframe;
P= 4;           %no. of TDOA microphone pairs

snr_db= 20;
snr= 10^(snr_db/10);
Ts= 1/fs;


%--- parameters associated with room impulse responses
c= 344;			%speed of sound
rlim= 5;			%represent (but not exactly) the no. of multipaths
room_dim= [ 3; 3; 2.5 ];    %room size
beta= [ 0.85 0.85 0.75; 0.85 0.85 0.5 ];	%reflection coefficients of walls
%beta= 1e-6*ones(2,3);

%speaker coordinate
x_s= zeros(3,L);
for el=1:L
    x_s(:,el)= [ 1.2; 1.8; 1.8 ]+ [ (el-1)*0.04; (el-1)*0.04; 0 ];
end;

x_sensor= zeros(3,2,P);     %microphone pair coordinates
x_sensor(:,1,1)= [ 1.25; 0.01; 1.8 ]; x_sensor(:,2,1)= [ 1.75; 0.01; 1.8 ];
x_sensor(:,1,2)= [ 0.01; 1.25; 1.8 ]; x_sensor(:,2,2)= [ 0.01; 1.75; 1.8 ];
x_sensor(:,1,3)= [ 1.25; 2.99; 1.8 ]; x_sensor(:,2,3)= [ 1.75; 2.99; 1.8 ];
x_sensor(:,1,4)= [ 2.99; 1.25; 1.8 ]; x_sensor(:,2,4)= [ 2.99; 1.75; 1.8 ];


dR= c*Ts;
max_d= ceil(.5/dR);       %max. no. of TDOA samples (.5 is the microphone pair spacing)


%---
h_len= 3000;      %length of room impulses responses (interpolated)

if exist(['roomir_',fcode,'.mat'],'file'), gen_ir= 'n'; 
else gen_ir= 'y'; end;

if gen_ir=='y',
    disp(['Computing the room impulse responses...']);
    h= zeros(h_len,2,P,L);
    for p=1:P
        for el=1:L
            h(:,1,p,el)= ir_room2(x_s(:,el)/dR,x_sensor(:,1,p)/dR,room_dim/dR,beta,h_len,rlim);
            h(:,2,p,el)= ir_room2(x_s(:,el)/dR,x_sensor(:,2,p)/dR,room_dim/dR,beta,h_len,rlim);
        end;
    end;
    save( ['roomir_',fcode], 'h' );
else
    load(['roomir_',fcode]);
end;

%--- load speech signals
s= wavread('SA1');	s= decimate(s,2); % desired speech
%s= randn(20*Nframe,1);

if length(s) > N,
    s= s(1:N); s= s/(norm(s)/sqrt(N));
else
    st= zeros(N,1); st(1:length(s))= s; 
    s= st; s= s/(norm(s)/sqrt(N));
end;
